//************************************************************************
//    Learn how to read the EEPROM data usine both PSV and Table read
//
//			Press SW5, the progrom will use the Table Read method
//          Press SW6, the progrom will use the PSV method
//************************************************************************

#include 	<p30fxxxx.h>
#include	<timer.h>
#include 	"C30EVM_LCD.h"			// Included the LCD function prototype

#define		Fosc 	7372800   		// Crystal frequency are 7.3728 MHz  
									
//---------------------------------------------------------------------------
// 			Regarding the Configuration bits declaration
// 			Please refer to p30F4011.h for more 
//---------------------------------------------------------------------------
   	_FOSC(CSW_FSCM_OFF & XT);   	 //	XT oscillator, Failsafe clock off
   	_FWDT(WDT_OFF);                  //	Watchdog timer disabled
  	_FBORPOR(PBOR_OFF & MCLR_EN);    //	Brown-out reset disabled, MCLR reset enabled
   	_FGS(CODE_PROT_OFF);             //	Code protect disabled

void Table_Read (unsigned int);
void Press_Key(void);

#define	SW5			PORTEbits.RE8		// Deine SW5
#define	SW6			PORTBbits.RB3
#define	DIR_SW5		TRISEbits.TRISE8 	// Define dierection for SW5
#define	DIR_SW6		TRISBbits.TRISB3	

const char LCD_MSG1[]="Press S5 (TABLE)";
const char LCD_MSG2[]="Press S6 ( PSV )";
const char LCD_MSG3[]="Using TAB Mode  ";
const char LCD_MSG4[]="Using PSV Mode  ";

const unsigned char _EEDATA(1) sine_table[ ] = 
//const unsigned char __attribute__((space(eedata),address(0x7fff00), aligned(1))) sine_table[ ] =
// Wŧi[JF EEPROM Address wb 0x7FFF00  
		{128,152,176,198,217,233,245,252,255,252 
		,245,233,217,198,176,152,128,102,78,56,37,21
   		,9,2,0,2,9,21,37,56,78,102};

unsigned char Sine_temp[8];
unsigned char i;
unsigned char j;
unsigned int EE_Index;
unsigned int EEPROM_Offset;
unsigned int *Addr_Point;
int debounce;
char SW_Input;

int main(void)
{
 	CORCONbits.PSV = 1 ;  				// Using the PSV read function
	ADPCFG=0xFFFF;
	DIR_SW5 = 1 ;
	DIR_SW6 = 1 ;
	OpenLCD( );							// Initial the LCD module for 4 bits Data mode and 5 * 7 Character 

	while(1)
	{	
		setcurLCD(0,0) ;				// Set LCD cursor at (0,0)
		putrsLCD( LCD_MSG1 ) ;			// Display LCD_MSG1 string on LCD
		setcurLCD(0,1) ;				// Set LCD cursor at (0,1)
		putrsLCD( LCD_MSG2 ) ;			// Display LCD_MSG2 string on LCD	

		Press_Key( );

		if (SW_Input=='5')
		{
			setcurLCD(0,0) ;				 
			putrsLCD( LCD_MSG3 ) ;			 
			SW_Input= 0x00;
			
			j=0;
			do
			{

				TBLPAG = __builtin_tblpage (sine_table) ;			// Set Sine_Table Page for Table Read
				EEPROM_Offset =  __builtin_tbloffset (sine_table) ; // get the Sine_Table offset address
				EEPROM_Offset= j * 8 + EEPROM_Offset ;				// Add the read block index

				Table_Read(EEPROM_Offset);							// Table Read function

				setcurLCD(15,0) ;
				putcLCD(j+0x30);									// Show the read block number on LCD line 1 
				setcurLCD(0,1) ;
				for (i=0;i<8;i++) puthexLCD (Sine_temp[i]); 		// Show the Hex on the LCD line 2

				Press_Key( );										// Wait for key press
				SW_Input=0x00;				
				j++;												// point to next 8 bytes on EEPROM
			}
			while(j<4);	
		}

		if (SW_Input=='6')
		{
			setcurLCD(0,0) ;								// Set LCD curser to (0,0)
			putrsLCD( LCD_MSG4 ) ;							// Display String "Using PSV Mode" on the LCD
			SW_Input= 0x00;									// Clear the key buffer
										 
			j=0;											// Set the EEPROM read block to zero (set one block have 8 bytes)
			do
			{
				asm ("push PSVPAG");						// Push PSVPAG for LCD ROM display message access
				PSVPAG= __builtin_psvpage (sine_table);		//get the PAVPAG to EEPROM ,(PSVPAG = 0xFF);			
				
				EE_Index = j * 8 ;  						// Set EEPRM index address	
				for (i=0; i<8;i++) Sine_temp[i] = (sine_table[EE_Index++]); //Read the EEPROM data from PSV RAM		
				
				asm ("pop PSVPAG");							// Pop PSVPAG for LCD ROM display message

				setcurLCD(15,0) ;
				putcLCD(j+0x30);							// Show the read block number on LCD line 1 
				setcurLCD(0,1) ;
				for (i=0;i<8;i++) puthexLCD (Sine_temp[i]); // Show the Hex on the LCD line 2

				Press_Key( );								// Wait for key press
				SW_Input=0x00;				
				j++;										// point to next 8 bytes on EEPROM
			}
			while(j<4);	
			
		} 
	}						
}

void Table_Read (unsigned int EE_Point)
{
				asm ("push.s") ;				 
				WREG1= EE_Point ;							// Use the WREG1 for the ROM source Index
				WREG2 = (int) &(Sine_temp[0]) ;				// Set the WREG2 for the destination RAM ARea Buffer
				asm ("repeat 	#3  			; repeat 4 time , 8 bytes");\
				asm	("tblrdl.w 	[W1++],[W2++] 	; Table Read data from ROM to RAM ");\
				asm ("pop.s");						 
}

void Press_Key(void)
{
	unsigned int i;

	while (1)
	{	
		if (debounce==0)					// Check the dubounce time is over?
		{
			if (!SW5)   					// debounce is completed, now checking the SW5 ststus
			{
				SW_Input = '5';				// The SW5 has been Press, Set the Key buffer = '5'
				debounce = 30;		 		// Set the debounce loop = 30	
				break;
			}
	
		 	if (!SW6) 						// Check the SW6 status
			{
				SW_Input = '6';				// The SW5 has been Press, Set the Key buffer = '5'
				debounce = 30;				// Set the debounce loop = 30
				break;
			}
		}
		else								// Process the debounce procedure
		{
			if (SW5 & SW6) 					// Bothe SW5 & SW6 ahve released ?
					{
					debounce--;				// Yes, decrement debounce loop once
					for (i=0;i<500;i++);	// Basic debounce delay time
					}
				else 						// Still have key press done
					debounce=30;			// Re-set the debounce loop = 30
		}		 	
	}

}
